#!/bin/bash
# This is special script for totaly removing edge-db
# It drops cluster, removes all directories of cluster, removes binfs and maintance cron file
# Used in debian package for postrm purge

pushd `dirname $0` > /dev/null

# Available params, ./purge.sh -?
_usage() {
    echo "Usage: $0 [-v version] [-c clustername] [-d datadir] [-l logdir] [-b binfsdir]"
    echo " NB! For default values see config/*.config!"
    echo "   -C, --config         Configuration name"
    echo "   -v, --version    PostgreSQL version"
    echo "   -c, --cluster    PostgreSQL cluster name"
    echo "   -P, --port       PostgreSQL port"
    echo "   -h, --host       PostgreSQL host"
    echo "   -d, --data-dir   PostgreSQL data directory"
    echo "   -l, --logs-dir   PosthreSQL logs directory"
    echo "   -b, --binfs-dir  BinFS data directory"
    echo "   -?, --help       This help text"
    exit 0
}

# Current dir
_ME=$(readlink -f "$0")
PWD=$(dirname "${_ME}")

# Config name
[ -f ${PWD}/.configure ] && CONFIG=$(cat ${PWD}/.configure | tr -d "\n\r")
CONFIG=${CONFIG:-default}

# CLI params
while test $# -gt 0; do
    case $1 in
        --config|-C)
            CONFIG="${2}"
            shift
        ;;
        --version|-v)
            PGVERSION="${2}"
            shift
        ;;
        --cluster|-c)
            PGCLUSTER="${2}"
            shift
        ;;
        --data-dir|-d)
            PGDATADIR="${2}"
            shift
        ;;
        --logs-dir|-l)
            PGLOGDIR="${2}"
            shift
        ;;
        --binfs-dir|-b)
            BINFSDIR="${2}"
            shift
        ;;
        --help|-?)
            _usage
            break
        ;;
        --)
            break
        ;;
        --*|-?)
            echo "Unknown option ${1}"
            exit 11
        ;;
        -*)
            split=$1
            shift
            set -- $(echo "$split" | cut -c 2- | sed 's/./-& /g') "$@"
            continue
        ;;
        *)
            echo "Unknown argument ${1}"
            exit 11
        ;;
    esac

    shift
done

# Configs
source $PWD/config/$CONFIG.config

# Helpers
_error(){
    echo "ERROR: $*" >&2
    kill $$
    exit -1
}

# Main body
if test "$EUID" -ne 0 ; then
    _error "Please run as root"
fi

# Defaults
# TODO - get default in postinst file and create cleanup.sh there
PGDPKG=`dpkg-query -l 'postgresql*' |grep '^ii' |cut -d " " -f 3 |grep -v "\-[a-z]" |xargs dpkg -s |grep Version |awk -F '[ +.]' '{print $2"."$3}'`
PGVERSION="${PGVERSION:-${PGDPKG%.*}}"
PGCLUSTER="${PGCLUSTER:-autocode}"
PGDATADIR="${PGDATADIR:-/var/opt/edge/pgdata/$PGVERSION/$PGCLUSTER}"
PGLOGDIR="${PGLOGDIR:-/var/log/edge/postgresql}"
BINFSDIR="${BINFSDIR:-/var/opt/edge/pgdata/binfs}"

# Droping PostgreSQL cluster
echo "Removing PostgreSQL cluster..."
CURRENT_CLUSTERS=$(pg_lsclusters | tail -n +2 | awk 'BEGIN{ str=""; }{ str=str$2" "str; }END{ print str; }')
if (echo "$CURRENT_CLUSTERS" | grep "$PGCLUSTER" > /dev/null); then
    pg_dropcluster --stop $PGVERSION $PGCLUSTER
    echo "Cluster dropped!"
fi

# Droping data and log dirs
if [ -d $PGDATADIR ]; then rm -rf $PGDATADIR; fi
if [ -d $PGLOGDIR ]; then rm -rf $PGLOGDIR; fi

# Droping pg-hba.conf
rm -rf /etc/postgresql/$PGVERSION/$PGCLUSTER/pg_hba.conf

# BinFS remove
echo "Removing BinFS..."
rm -rf "${BINFSDIR}"


echo "ALL DONE!"
