-- {
--  frame: == string
--  film: == string
--  filmts: == bigint
--  ctime: == timestamptz
--  owner: == string
--  job: == bigint
--  detector: == string
--  rect: == [l t r b]
--  hash: == hex string
--  attr: == json
-- }



create or replace function fb.fb_add ( idata jsonb ) returns void as $$
declare 
    iid bigint;
begin
    insert into fb.frames (frame, film, filmts, ctime, owner)
        values(
            coalesce(idata ->> 'frame',''),
            coalesce(idata ->> 'film',''),
            case when idata ->> 'filmts' isnull or idata ->> 'filmts' = '' then 0 else (idata ->> 'filmts')::bigint end,
            case when idata ->> 'ctime' isnull or idata ->> 'ctime' = '' then now() else (idata ->> 'ctime')::timestamptz end,
            idata ->> 'owner'
        ) 
        on conflict (frame,film,filmts)
        do update
            set frame = excluded.frame
        returning id into iid;    
    insert into fb.detections (frameid, jobid, detector, rect, hash, attr)
        values(
            iid,
            case when idata ->> 'job' isnull or idata ->> 'job' = '' then null else (idata ->> 'job')::uuid end,
            idata ->> 'detector',
            box(point((idata->'rect'->>0)::float,(idata->'rect'->>1)::float),point((idata->'rect'->>2)::float,(idata->'rect'->>3)::float)),
            decode(idata ->> 'hash','hex'),
            idata -> 'attr'
        );
end;
$$ language plpgsql;



create or replace function fb.fb_add_csv( iline text, idelimiter varchar(1) default '|' ) returns void as $$
import csv 
import json
import os
import StringIO

if os.path.isfile(iline):
    f = open(iline,'rt')
else:
    f = StringIO.StringIO(iline)
p = plpy.prepare('select fb.fb_add($1)',['jsonb'])    
for r in csv.reader(f,delimiter=idelimiter):
    if len(r) < 13:
        raise ValueError('Invalid string format')
        return
    d = {'job':r[0],'owner':r[1],'frame':r[2],'film':r[3],'filmts':r[4],'ctime':r[5],'detector':r[6],'rect':[r[7],r[8],r[9],r[10]],'hash':r[11],'attr':json.loads(r[12])}
    plpy.execute(p,[json.dumps(d)],0)
$$ language plpython3u;


