
create or replace function hwp.watchload (
    iowner varchar(128),
    iident varchar(128),
    itemplate text,
    itemplatename varchar(32),
    iload text,
    isha1 varchar(40),
    idescription text,
    ioperator varchar(128),
    inneraction varchar(32) default 'none',
    crossaction varchar(32) default 'set',
    outeraction varchar(32) default 'set',
    out total int,
    out inserted int,
    out deleted int,
    out updated int,
    out errors json
) as $$
#-*-  coding:utf-8 -*-
import json, re, codecs, os, sys, hashlib, uuid, traceback
from uuid import uuid4
_res = {
    'total': 0,
    'inserted': 0,
    'deleted': 0,
    'updated': 0,
    'errors': [] 
}
_ainner = inneraction.lower()
_aouter = outeraction.lower()
_across = crossaction.lower()
_isha1 = isha1.lower() if isha1 else None
_rev = uuid.uuid1().hex

if len(iowner or '') * \
    len(iident or '') * \
    (len(itemplate or '') + len(itemplatename or '')) * \
    len(iload or '') == 0:
    plpy.error("Null string argument", hint='400')    

if  not _ainner in ['set','unset','none'] or \
    not _aouter in ['set','unset','none'] or \
    not _across in ['set','unset','none'] :
    plpy.error('Invalid action', hint='400')

if not itemplate:
    rv = plpy.execute("select template from hwp.watchlistpl where ident = %s" % (plpy.quote_literal(itemplatename)),1)
    if rv.nrows() > 0:
        _template = json.loads(rv[0]['template'])
    else:
        plpy.error("Template not found", hint='404')
else:
    _template = json.loads(itemplate)

if _template \
    and 'fields' in _template \
    and 'plate' in _template['fields'] \
    and 'format' in _template['fields']['plate']:
    pass
else:
    plpy.error("Invalid format", hint='400')
if not 'rowsep' in _template: _template['rowsep'] = '[\r\n]+'
if not 'colsep' in _template: _template['colsep'] = ','


rv = plpy.execute("select * from hwp.watchlist where owner = %s and ident = %s" % (
    plpy.quote_literal(iowner),
    plpy.quote_literal(iident)),1)
_modify = False
if rv.nrows() > 0:
    _modify = True
    if not rv[0]['active']:
        plpy.error("List should be active", hint='404')

_sid = "hwpwl_"+uuid4().hex

plpy.execute("create temp table %s (like hwp.watchlistvcs); alter table %s alter op drop not null; alter table %s alter optime set default now();create unique index i_%s_plate on %s using btree(plate);"%(_sid,_sid,_sid,_sid,_sid))
_op = -1 if _ainner == 'unset' else None

_h = hashlib.sha1()
if os.path.isfile(iload):
    with open(iload,'rb') as f:
        while True:
            _d = f.read(65536)
            if not _d: break
            _h.update(_d)
else:
    _h.update(iload.encode())
_sha1 = _h.hexdigest()

if _isha1:
    if _isha1 != _sha1:
        plpy.error("SHA1 doesn`t match", hint='400')

rv = plpy.execute("select sha1,inneraction,crossaction,outeraction from hwp.watchlistlog where owner = %s and ident = %s order by revision desc limit 1" % (
    plpy.quote_literal(iowner),
    plpy.quote_literal(iident)) ,1)
if rv.nrows() > 0:
    if _sha1 == rv[0]['sha1'] and _ainner == rv[0]['inneraction'] and _across == rv[0]['crossaction'] and _aouter == rv[0]['outeraction']:
        return _res

plpy.execute("insert into %s (owner,ident,plate,class,color,make,model,alarm,memo,meta,revision,op) select *,%s,%s from hwp.watchlistitems where owner = %s and ident = %s"%(
    _sid, 
    plpy.quote_literal(_rev),  
    _op if _op else 'null',
    plpy.quote_literal(iowner),
    plpy.quote_literal(iident)
))

def __process(no,line):
    _res['total'] += 1
    line = line.strip().rstrip('\r')
    if 'validators' in _template:
        if 'negative' in _template['validators'] and _template['validators']['negative']:
            for v in _template['validators']['negative']:
                if re.match(v,line, re.UNICODE):
                    _res['errors'].append([no,line,"Negative validator"])
                    return
        if 'positive' in _template['validators'] and _template['validators']['positive']:
            _match = False
            for v in _template['validators']['positive']:
                if re.match(v,line, re.UNICODE):
                    _match = True 
                    break
            if not _match:
                _res['errors'].append([no,line,"Positive validator"])
                return
    if not _template['colsep'] in line and not re.match('^\w+$', line, re.UNICODE):
        _res['errors'].append([no,line,"No columns separated"])
        return
    try:
        ar = re.split(_template['colsep'],line,re.UNICODE)
        d = {}
        for key in _template['fields'].keys():
            f = _template['fields'][key]['format'].format(*ar)
            if 'sub' in _template['fields'][key]:
                for s in _template['fields'][key]['sub']:
                    f = re.sub(s[0],s[1],f)
            d[key.encode('utf-8')] = f.encode('utf-8')
        if 'meta' in _template and _template['meta']:
            _m = []
            for m in _template['meta']:
                _v = m['format'].format(*ar)
                if 'sub' in m:
                    for s in m['sub']:
                        _v = re.sub(s[0],s[1],_v)
                _m.append({
                    'name': m['name'],
                    'display': m['display'],
                    'value': _v.encode('utf-8')
                })
            if _m:
                d[u'meta'] = _m
        if d:    
            _s = ','.join([(u"%s='%s'"%(k,(json.dumps(v).decode('unicode-escape') if isinstance(v,list) else v.decode('utf-8')))).encode('utf-8') for k,v in d.items() if not k==u'plate'])
            if _s: _s = ','+_s
            if _aouter == 'none':
                rv = plpy.execute("update %s set op=%s%s where plate = %s" % (
                    _sid, (0 if _across == 'set' else -1 if _across == 'unset' else 'null'), _s,
                    plpy.quote_literal(d['plate'])
                ))
            else:
                plpy.execute("insert into %s (op,optime,owner,ident,revision,%s) values(%s,now(),%s,%s,%s,%s) on conflict (plate) do update set op=%s%s"%(
                    _sid, 
                    ','.join(d.keys()).encode('utf-8'),
                    (1 if _aouter == 'set' else (-1 if _aouter == 'unset' else 'null')),
                    plpy.quote_literal(iowner),
                    plpy.quote_literal(iident),
                    plpy.quote_literal(_rev),
                    ','.join([plpy.quote_literal(json.dumps(v).decode('unicode-escape').encode('utf-8') if isinstance(v,list) else v) for v in d.values()]),
                    (0 if _across == 'set' else -1 if _across == 'unset' else 'null'), _s
                ))

        else:
            _res['errors'].append([no,line,"No fields found"])
    except Exception as  e:
        _res['errors'].append([no,line,"ERROR:"+str(e)])
        pass


if os.path.isfile(iload):
    with codecs.open(iload,"r","utf-8") as f:
        for i,line in enumerate(f):
            line = line.strip().rstrip('\r').replace('"', '\\"')
            if line:
                __process(i,line)
else:
    if not _template['rowsep'] in iload and not _template['colsep'] in iload and os.sep in iload:
        plpy.error("Seems to be a file name but not exists", hint='404')
    else:    
        for i,line in enumerate(re.split(_template['rowsep'],iload)):
            __process(i,line.strip())

rv = plpy.execute("select * from hwp.watchsweep(%s,%s,%s)" % (plpy.quote_literal(_sid),plpy.quote_literal(_sha1),plpy.quote_literal(_rev)),1)
if rv.nrows() > 0:
    _res['inserted'] = rv[0]['inserted']
    _res['deleted']  = rv[0]['deleted']
    _res['updated'] = rv[0]['updated']
    
    plpy.execute("insert into hwp.watchlistlog (moment,owner,ident,revision,sha1,template,inneraction,crossaction,outeraction,description,operator,inserted,updated,deleted)values(now(),%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s)" % (
        plpy.quote_literal(iowner),
        plpy.quote_literal(iident),
        plpy.quote_literal(_rev),
        plpy.quote_literal(_sha1) if _sha1 else "'null'",
        plpy.quote_literal(itemplatename),
        plpy.quote_literal(_ainner),plpy.quote_literal(_across),plpy.quote_literal(_aouter),
        plpy.quote_literal(idescription),
        plpy.quote_literal(ioperator),
        _res['inserted'],
        _res['updated'],
        _res['deleted']
    ),0)
    
    if _modify:
        plpy.execute("update hwp.watchlist set utime = now() where owner = %s and ident = %s" % (
            plpy.quote_literal(iowner),
            plpy.quote_literal(iident)
        ))
    else:
        plpy.execute("insert into hwp.watchlist (owner,ident) values(%s,%s)"%(
            plpy.quote_literal(iowner),
            plpy.quote_literal(iident)
        ))


_res['errors'] = json.dumps(_res['errors']).decode('unicode-escape').encode('utf-8')
return _res
$$ language plpython3u;




