CREATE or replace FUNCTION lib.levenstein(ifirst text, isecond text, iinsertcost double precision DEFAULT 1.0, ideletecost double precision DEFAULT 1.0, ichangecost double precision DEFAULT 1.0) RETURNS double precision
    LANGUAGE plpgsql
    AS $$
declare
	afirst	varchar[];
	asecond	varchar[];
	distance  double precision[][];
	nfirst	integer;
	nsecond	integer;
	cost	double precision;
	res		double precision;
	i 		integer;
	j		integer;
begin

	nfirst  := character_length(ifirst);	
	nsecond := character_length(isecond);	
	
	if nfirst  = 0 then return nsecond*iinsertcost; end if; 
	if nsecond = 0 then return nfirst *ideletecost; end if; 
	
	select into distance (E'{'||string_agg((select array_agg(0) from generate_series(1,nsecond+1) )::text,',')||E'}')::integer[][] from generate_series(1,nfirst+1);
	
	distance[1][1] := 0.0;

	for i in 1..nfirst loop
		afirst := array_append(afirst,substring(ifirst from i for 1)::varchar);
	end loop;	
	for j in 1..nsecond loop
		asecond := array_append(asecond,substring(isecond from j for 1)::varchar);
	end loop;

	for i in 1..nfirst+1 loop
		distance[i][1] := (i-1) * iinsertcost; 
	end loop;
	
	for j in 2..nsecond+1 loop
		distance[1][j] := (j-1) * ideletecost;
	end loop;	
	
	for i in 1..nfirst loop
		for j in 1..nsecond loop
			if afirst[i] = asecond[j] then
				cost := 0;
			else
				cost := ichangecost;
			end if;
			select into res min(x) from unnest(array[
				distance[i][j+1] + ideletecost,
				distance[i+1][j] + iinsertcost,
				distance[i][j] + cost
			]) as x; 
			distance[i+1][j+1] := res;
			if i>1 and j>1 and afirst[i] = asecond[j-1] and afirst[i-1] = asecond[j] then
				select into res min(x) from unnest(array[
					distance[i+1][j+1],
					distance[i-1][j-1] + cost
				]) as x; 
				distance[i+1][j+1] := res;
			end if;
		end loop;
	end loop;
	return distance[nfirst+1][nsecond+1];
end;
$$;
