CREATE or replace FUNCTION lib.recrop(i_media bytea, i_rect integer[], i_margin double precision[], i_sizes integer[], i_returnrect boolean DEFAULT false) RETURNS public.cropresult
    LANGUAGE plpython3u
    AS $$
import cStringIO

from PIL import Image, ImageDraw

try:
    ims = Image.open(cStringIO.StringIO(i_media))
    (ims_w, ims_h) = ims.size
except Exception as ex:
    plpy.execute("select core.log('lib.recrop','error','{0}')".format(ex), 0)
    return None

if len(i_sizes) == 4:
    (fw, fh, frw, frh, faw, fah) = (i_sizes[0], i_sizes[1], i_sizes[2],
                                    i_sizes[3], None, None)
elif len(i_sizes) == 6:
    (fw, fh, frw, frh, faw, fah) = (i_sizes[0], i_sizes[1], i_sizes[2],
                                    i_sizes[3], i_sizes[4], i_sizes[5])
else:
    plpy.execute("select core.log('lib.recrop','error','invalid sizes')", 0)
    return None

(fw, fh, frw, frh, faw, fah) = (
    fw if (fw is not None and fw > 0) else None,
    fh if (fh is not None and fh > 0) else None,
    frw if (frw is not None and frw > 0) else None,
    frh if (frh is not None and frh > 0) else None,
    faw,
    fah
)

(rl, rt, rr, rb) = (i_rect[0], i_rect[1], i_rect[2], i_rect[3])
(rl, rt, rr, rb) = (
    0 if (rl is None or rl < 1 or rl > ims_w or rl >= rr) else rl,
    0 if (rt is None or rt < 1 or rt > ims_h or rt >= rb) else rt,
    0 if (rr is None or rr < 1 or rr > ims_w or rr <= rl) else rr,
    0 if (rb is None or rb < 1 or rb > ims_h or rb <= rt) else rb
)

##dr = ImageDraw.Draw(ims)
##dr.rectangle(((rl,rt),(rr,rb)), outline = "red")

if len(i_margin) == 4:
    (lmcw, lmch, tmcw, tmch, rmcw, rmch, bmcw, bmch) = (i_margin[0], 0.0,
                                                        i_margin[1], 0.0,
                                                        i_margin[2], 0.0,
                                                        i_margin[3], 0.0)
elif len(i_margin) == 8:
    (lmcw, lmch, tmcw, tmch, rmcw, rmch, bmcw, bmch) = (
        i_margin[0],
        i_margin[1],
        i_margin[2],
        i_margin[3],
        i_margin[4],
        i_margin[5],
        i_margin[6],
        i_margin[7]
    )
else:
    plpy.execute("select core.log('lib.recrop','error',"
                 "'invalid margin sizes')", 0)
    return None

(lmcw, lmch, tmcw, tmch, rmcw, rmch, bmcw, bmch) = (
    0 if (lmcw is None) else lmcw,
    0 if (lmch is None) else lmch,
    0 if (tmcw is None) else tmcw,
    0 if (tmch is None) else tmch,
    0 if (rmcw is None) else rmcw,
    0 if (rmch is None) else rmch,
    0 if (bmcw is None) else bmcw,
    0 if (bmch is None) else bmch
)

if ((rr - rl) < 1 or (rr - rl) > ims_w - 1 or
        (rb - rt) < 1 or (rb - rt) > ims_h - 1):
    plpy.execute("select core.log('lib.recrop','error',"
                 "'invalid rect borders')", 0)
    return None

sw = (rr - rl)
sh = (rb - rt)

lm = int(sw * lmcw + sh * lmch)
rm = int(sw * rmcw + sh * rmch)
tm = int(sw * tmcw + sh * tmch)
bm = int(sw * bmcw + sh * bmch)

bw = sw + lm + rm
bh = sh + tm + bm
kb = bw * 1.0 / bh

if bw < 1 or bh < 1:
    plpy.execute("select core.log('lib.recrop','error',"
                 "'invalid body borders')", 0)
    return None

adapt = False

if faw or fah:
    adapt = True
    if faw == 0:
        faw = bw
    if fah == 0:
        fah = bh
    fw = faw
    fh = fah

if (fw and (fw < 2 or fw > ims_w - 2)) or (fh and (fh < 2 or fh > ims_h - 2)):
    plpy.execute("select core.log('lib.recrop','error',"
                 "'invalid body sizes')", 0)
    return None

kr = None
if frw:
    kr = frw * 1.0 / sw
if frh:
    kr = frh * 1.0 / sh
if frw and frh:
    kr = max(frw * 1.0 / sw, frh * 1.0 / sh)

if fw and fh:
    kf = fw * 1.0 / fh
    if not kb == kf:
        if bw >= bh:
            lm = (int(round(lm * (1 - (bw - bh * kf) * 1.0 / (lm + rm))))
                  if lm > 0 else 0)
            rm = (int(round(rm * (1 - (bw - bh * kf) * 1.0 / (lm + rm))))
                  if rm > 0 else 0)
        else:
            tm = (int(round(tm * (1 - (bh - bw * 1.0 / kf) * 1.0 / (tm + bm))))
                  if tm > 0 else 0)
            bm = (int(round(bm * (1 - (bh - bw * 1.0 / kf) * 1.0 / (tm + bm))))
                  if bm > 0 else 0)
bw = sw + lm + rm
bh = sh + tm + bm
kb = bw * 1.0 / bh

ka = min((ims_w - 1) * 1.0 / bw, (ims_h - 1) * 1.0 / bh)
if ka < 1:
    lm = int(round(lm * (1 - (bw * (1.0 - ka)) / (lm + rm)))) if lm > 0 else 0
    rm = int(round(rm * (1 - (bw * (1.0 - ka)) / (lm + rm)))) if rm > 0 else 0
    tm = int(round(tm * (1 - (bh * (1.0 - ka)) / (tm + bm)))) if tm > 0 else 0
    bm = int(round(bm * (1 - (bh * (1.0 - ka)) / (tm + bm)))) if bm > 0 else 0

bw = sw + lm + rm
bh = sh + tm + bm
kb = bw * 1.0 / bh

(cl, ct, cr, cb) = (
    rl - lm,
    rt - tm,
    rr + rm,
    rb + bm
)

xo = yo = 0
if adapt:
    if cl < 0:
        cr -= cl
        cl = 0
    if ct < 0:
        cb -= ct
        ct = 0
    if cr > ims_w - 1:
        cl = cl - cr + ims_w - 1
        cr = ims_w - 1
    if cb > ims_h - 1:
        ct = ct - cb + ims_h - 1
        cb = ims_h - 1
else:
    if cl < 0:
        xo = -cl
        cl = 0
    if ct < 0:
        yo = -ct
        ct = 0
    if cr > ims_w - 1:
        cr = ims_w - 1
    if cb > ims_h - 1:
        cb = ims_h - 1

imd = Image.new("RGB", (bw, bh), (0, 0, 0))
imp = ims.crop((cl, ct, cr, cb))
imd.paste(imp, (xo, yo))
(ww, wh) = (imd_w, imd_h) = imd.size

if fw and not fh:
    wh = int(round(wh * fw * 1.0 / ww))
    ww = fw
if fh and not fw:
    ww = int(round(ww * fh * 1.0 / wh))
    wh = fh
if fw and fh:
    (ww, wh) = (fw, fh)

if kr:
    kw = max(ww * 1.0 / imd_w, wh * 1.0 / imd_h)
    if kr > kw:
        ww = int(round(ww * (1 + kr - kw)))
        wh = int(round(ww * (1 + kr - kw)))

if ww != imd_w or wh != imd_h:
    imd = imd.resize((ww, wh), Image.BICUBIC)

t = cStringIO.StringIO()
imd.save(t, "JPEG")
plate = t.getvalue()
t.close()
if i_returnrect:
    tr = '(%(a)f , %(b)f , %(c)f , %(d)f , %(x)f , %(y)f)' % {"a": cl,
                                                              "b": ct,
                                                              "c": cr,
                                                              "d": cb,
                                                              "x": -100000,
                                                              "y": -100000}
else:
    tr = '(%(a)f , %(b)f , %(c)f , %(d)f , %(x)f , %(y)f)' % {
        "a": ((imd_w * 1.0) / (ww * 1.0)), "b": 0.0, "c": 0.0,
        "d": ((imd_h * 1.0) / (wh * 1.0)), "x": (cl - xo), "y": (ct - yo)}
return {"data": plate, "transform": tr}
$$;
