CREATE or replace FUNCTION lib.subtitle(imedia bytea, iconf text, itemplate text, irect text, idata text) RETURNS bytea
    LANGUAGE plpython3u
    AS $$
import cStringIO
from PIL import Image, ImageChops, ImageDraw, ImageFont

try:
    ims = Image.open(cStringIO.StringIO(imedia))
    (ims_w, ims_h) = ims.size
except Exception as ex:
    plpy.execute("select core.log('lib.subtitle','error','{0}')"
                 .format(str(ex).replace("'", "`")), 0)
    return None
_DEFAULTCONF = {
    "float": "bottom",
    "height": 0,
    "grid": (3, 3),
    "align": ("left", "center"),
    "padding": (3, 0),
    "font": {"name": "cour.ttf", "size": 14}
}
try:
    conf = eval(iconf)
    template = eval(itemplate)
    data = eval(idata)
    rects = eval(irect)
except Exception as ex:
    plpy.execute("select core.log('lib.subtitle parser','error','{0}')"
                 .format(str(ex).replace("'", "`")), 0)
    return None

if 'float' not in conf:
    conf['float'] = _DEFAULTCONF['float']
if 'height' not in conf:
    conf['height'] = _DEFAULTCONF['height']
if 'grid' not in conf:
    conf['grid'] = _DEFAULTCONF['grid']
if 'align' not in conf:
    conf['align'] = _DEFAULTCONF['align']
if 'padding' not in conf:
    conf['padding'] = _DEFAULTCONF['padding']
if 'font' not in conf:
    conf['font'] = _DEFAULTCONF['font']

try:
    font = ImageFont.truetype(conf['font']['name'], conf['font']['size'],
                              encoding='unic')
except Exception as ex:
    plpy.execute("select core.log('lib.subtitle','error','{0}')"
                 .format(str(ex).replace("'", "`")), 0)
    return None

(fw, fh) = font.getsize('0987654321')

if conf['height'] == 0:
    conf['height'] = fh * 2 * conf['grid'][0]

(dx, dy) = (int(ims_w // conf['grid'][1]),
            int(conf['height'] // conf['grid'][0] // 2))

imt = Image.new("RGB", (ims_w, conf['height']), (0, 0, 0))
draw = ImageDraw.Draw(imt)

for tpl in template:
    if tpl[4] in data:
        txt = tpl[2] + ' ' + data[tpl[4]] + tpl[3]
        (x, y) = (tpl[1] * dx + conf['padding'][0],
                  ((tpl[0] * 2 + 1) * dy - int(fh // 2)))
        plpy.notice('DT: ' + str((x, y)) + txt)
        draw.text((x, y), txt.decode('utf-8'), font=font)

imr = Image.new("RGB", (ims_w, ims_h + conf['height']), (0, 0, 0))

draw = ImageDraw.Draw(ims)
for rect in rects:
    draw.rectangle(list(rect), outline='white')
    draw.rectangle([a + b for a, b in zip(list(rect), [1, 1, -1, -1])],
                   outline='white')

if conf['float'] == 'bottom':
    imr.paste(ims, (0, 0))
    imr.paste(imt, (0, ims_h))
else:
    imr.paste(imt, (0, 0))
    imr.paste(ims, (0, conf['height'] + 1))

t = cStringIO.StringIO()
imr.save(t, "JPEG")
res = t.getvalue()
return res
$$;
