create or replace function public._estimatch( idata json, ipattern json, out ematch boolean, out elen integer ) as $$
import json, operator, re
pattern = json.loads(ipattern) if ipattern else dict()
data = json.loads(idata) if idata else dict()
elen = 0
ematch = True
operators = {
    'lt': operator.lt,
    '<' : operator.lt,
    'le': operator.le,
    '<=': operator.le,
    'eq': operator.eq,
    '==': operator.eq,
    'ne': operator.ne,
    '<>': operator.ne,
    '!=': operator.ne,
    'ge': operator.ge,
    '>=': operator.ge,
    'gt': operator.gt,
    '>' : operator.gt,
    '~' : re.match
}
for attr,body in pattern.items():
    if type(body) != list or len(body) != 2:
        plpy.error('Invalid pattern body for ' + attr, hint='400')
    [op,val] = body
    if not op in operators:
        plpy.error('Invalid pattern body operator ' + op, hint='400')
    if attr in data:
        o = operators[op]
        if o.__module__ in ['operator','_operator']:
            res = o(data[attr], val)
        elif o.__module__ == 're':
            res = o(val, data[attr]) is not  None
        else:
            res = False
        if res == True:
            elen += 1
        else:
            ematch = False
            break
    else:
        ematch = False
        break
return {"ematch": ematch, "elen": elen}    
$$ language plpython3u;

create or replace function public.estimate_patternclause( ipattern json ) returns text as $$
import json
__ops = {
    'lt': '<',
    '<' : '<',
    'le': '<=',
    '<=': '<=',
    'eq': '=',
    '==': '=',
    'ne': '!=',
    '<>': '!=',
    '!=': '!=',
    'ge': '>=',
    '>=': '>=',
    'gt': '>',
    '>' : '>',
    '~' : '~'
}    
res = "true"    
pattern = json.loads(ipattern) if ipattern else dict()
for k,v in pattern.items():
    rv = plpy.execute("""select estimate('eventkeys','{"key":"%s"}') as key""" % k)
    if rv.nrows() > 0:
        _key = rv[0]['key'] if rv[0]['key'] else k
    else:
        _key = k
    _op = __ops[v[0]] if v[0] in __ops else '=' 
    _val = v[1]
    res += " and %s %s '%s'" % (_key, _op, _val)
return res    
$$ language plpython3u;

create or replace function public.estimate( iname varchar, idata json default null ) returns varchar as $$
    select 
        value 
    from 
        public.estimations,_estimatch($2,pattern) 
    where 
        name = $1 and ematch 
    order by elen desc limit 1;
$$ language sql immutable;

create or replace function public.estimate_details( iname varchar, idata json default null, out pattern json, out value text, out elen integer ) returns setof record as $$
    select 
        pattern, value, elen
    from 
        public.estimations,_estimatch($2,pattern) 
    where 
        name = $1 and ematch 
    order by elen desc;
$$ language sql immutable;

create or replace function public.api_getestimate(idata jsonb, out errcode integer, out errmsg varchar, out result jsonb) as $$
declare
    sec text;
begin
    errcode := 0;
    errmsg := 'Ok';
    sec := idata ->> 'section';
    select into result jsonb_agg(jsonb_build_object('pattern',pattern,'value',value)) from public.estimations where name = sec;    
    if not found then
        errcode := 1;
        errmsg := 'Section not found';
    end if;    
exception when others then
errcode := -10;
errmsg := format('%s: %s',SQLSTATE,to_jsonb(SQLERRM));
end;
$$ language plpgsql;

create or replace function public.api_setestimate(idata jsonb, out errcode integer, out errmsg varchar, out result jsonb) as $$
declare
    sec text;
    data jsonb;
    rec jsonb;
begin
    errcode := 0;
    errmsg := 'Ok';
    sec := idata ->> 'section';
    data := idata -> 'data';
    if sec isnull or data isnull then
        errcode := 1;
        errmsg := 'Invalid data';
    end if;
    delete from public.estimations where name = sec;
    for rec in select jsonb_array_elements(data) loop
        insert into public.estimations 
            (name, pattern, value)
        values (
            sec,
            rec -> 'pattern',
            rec ->> 'value'
        );  
    end loop;    
exception when others then
errcode := -10;
errmsg := format('%s: %s',SQLSTATE,to_jsonb(SQLERRM));
end;
$$ language plpgsql;

