#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# (k) made-on-the-knee-of /dragon, 2017

"""

UNIT LIST

== API

- api_discovery
    -- origin
    -- lane
    -- string
    -- opt
    -- const
    -- estimate

- api_plateattrget    

- api_report_get

- api_filter

"""

import eventor
import json, os, sys, time
from datetime import datetime, timedelta
import psycopg2
import psycopg2.extras
import psycopg2.extensions
import importlib
psycopg2.extensions.register_type(psycopg2.extensions.UNICODE)
psycopg2.extensions.register_type(psycopg2.extensions.UNICODEARRAY)

#-------------------

def _usage():
    print("""

Usage: %s <command> <arg...>
- discovery origin
- discovery profile
- discovery lane ['{"origin":"..."}']    
- discovery string ['{"section":"..", "lang":"..."}']
- discovery opt ['{"section":"...", "keys:"...", ""lang":"..."}']
- discovery const ['{"kind":"...", "section":"..."}']
- filter ['{"limit":...,"order":"...","by":"...","event":{.....}}']
- filter-feed ['{"limit":...,"order":"...","ref-ts-order":...']
- filter-arch ['{"limit":...,"order":"...","ref-db-order":...']
- filter import <filename>
""" % (sys.argv[0])) 

#-------------------

importlib.reload(sys)
sys.setdefaultencoding('utf8')

dbconn = psycopg2.connect( "host='localhost' port=5432 dbname='av4' user='autocode' password='autocode'" )
dbconn.set_isolation_level(psycopg2.extensions.ISOLATION_LEVEL_AUTOCOMMIT)

dbc = dbconn.cursor(cursor_factory=psycopg2.extras.DictCursor)

if len(sys.argv) < 2:
    _usage()
    sys.exit(0)

cmd = sys.argv[1]

if cmd == 'discovery':
    if len(sys.argv) < 3:
        _usage()
        sys.exit(0)
    arg = sys.argv[2]
    param = {}
    if len(sys.argv) > 3:
        param = json.loads(sys.argv[3])
    param = ",".join(["'%s','%s'"%(k,v) for k,v in param.items()])
    if param: param = ','+param
    sql = """
    select * from api_discovery(
        jsonb_build_object(
            %s
        )
    )""" % (arg , param)
elif cmd == 'filter':
    param = {'@header':{"class": "vitml.edge.api", "action": "filter", "object": "event", "vendor": "VIT", "version": "0.1"}}
    if len(sys.argv) > 2:
        _param = sys.argv[2]
        if -param == 'import':
            if len(sys.argv) < 3:
                _usage()
                sys.exit(0)
            else:
                fn = sys.argv[3]
                if os.path.isfile(fn):
                    with open(fn,"rt") as f:
                        _param = json.load(f)
                else:
                    print("Import file not found: " + fn)
                    sys.exit(0)
        else:
            _param = json.loads(_param)
        param.update(_param)    
    else:
        _usage()
        sys.exit(0)
    sql = """    
        select * from api_event_filter (
            '%s'::jsonb
    )""" % (json.dumps(param))
elif cmd == 'filter-feed':
    param = {'@header':{"class": "vitml.edge.api", "action": "filter", "object": "event", "vendor": "VIT", "version": "0.1"}}
    if len(sys.argv) > 2:
        _param = sys.argv[2]
        _param = json.loads(_param)
        param.update(_param)    
    else:
        _usage()
        sys.exit(0)
    sql = """    
        select * from api_event_filter_feed (
            '%s'::jsonb
    )""" % (json.dumps(param))
elif cmd == 'filter-arch':
    param = {'@header':{"class": "vitml.edge.api", "action": "filter", "object": "event", "vendor": "VIT", "version": "0.1"}}
    if len(sys.argv) > 2:
        _param = sys.argv[2]
        _param = json.loads(_param)
        param.update(_param)    
    else:
        _usage()
        sys.exit(0)
    sql = """    
        select * from api_event_filter_arch (
            '%s'::jsonb
    )""" % (json.dumps(param))
else:
    _usage()
    sys.exit(0)

if sql:
    print("QUERY:\n---------------\n"+sql+"\n")
    dbc.execute(sql) 
    r = dbc.fetchone()
    print("RESULT:\n---------------\nCODE: %s\nMSG: %s\n%s" % (r[0],r[1],json.dumps(r[2],indent=4,sort_keys=True) if (r[2]) else ''))

