#!/usr/bin/python3

import sys, os
import re
from glob import glob
from datetime import datetime, timedelta
from distutils.util import strtobool

def prompt(query):
	sys.stdout.write('%s [y/n]: ' % query)
	val = input()
	try:
		ret = strtobool(val)
	except ValueError:
		sys.stdout.write('Please answer with a y/n\n')
		return prompt(query)
	return ret

if len(sys.argv) < 3:
	print("Usage: "+sys.argv[0]+" <binfsdir> <offset> [minkey]")
	exit(0)

root = offset = minkey = None

try:
	root = sys.argv[1]
	if not os.path.isdir(root): raise ValueError('Invaid storage directory')
	offset = int(sys.argv[2])
	if len(sys.argv) == 4: minkey = sys.argv[3]
except Exception as exc:
	print("Argument error: "+str(exc))
	exit(0)

print("\n")
print("ATTENTION!")
print("You ar going to shift the binfs storage")
print("\t"+root)
print("which contains the ")
for d in glob(os.path.join(root,'*')): print("\t"+d)
print("FOR %s HOURS" % (offset))
print("----------------------------------------")
if not prompt("Confirm it? "): exit(0)
print("\n\n")

for _r, _d, _f in os.walk(root):
	if _f:
		[_t,_hour] = os.path.split(_r)
		[_t,_day]  = os.path.split(_t)
		[_t,_month]  = os.path.split(_t)
		[_base,_year]  = os.path.split(_t)
		for _file in sorted(_f):
			if (minkey is None or (_minkey and _file > minkey)) and re.match(r'^[0-9]+\.[0-9]+$',_file):
				d = datetime(int(_year),int(_month),int(_day),int(_hour),0,0) + timedelta(hours=offset)
				_npath = os.path.join(_base,str(d.year),str(d.month).zfill(2),str(d.day).zfill(2),str(d.hour).zfill(2))
				if not os.path.isdir(_npath): os.makedirs(_npath,0o755)
				print(_r+" : "+_file, end=' ')
				os.rename( os.path.join(_r,_file), os.path.join(_npath,_file))
				print("...\t[Ok]")
		print("Completed.")
		print("It is strongly recommended to perform binfs.restat() now!")
	
